#!/usr/bin/env python
# encoding: utf-8

"""pywebhdfs: python Web HDFS Client."""

from .client import WebHDFSClient
import logging as lg

__version__ = '1.0.1'
__author__  = 'Yassine Azzouz'
__license__ = 'MIT'
